--[[
************************************************************************

options.lua

File date: 2019-11-24T10:10:35Z
File revision: 
Project revision: 275
Project version: r275

Author: Angelike

************************************************************************
--]]

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")

-- Init constants
local MODNAMESTRING = ... -- AddonName

local getColor, setColor;
local validateSetGet, getSetValue, checkHandler;
local MainOptionsTable = {}

local L = LibStub("AceLocale-3.0"):GetLocale("beql")

-- local AceConfig = LibStub("AceConfig-3.0")
local AceConfigReg = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

if (not beql.IsDisabled) then
    function beql:IsDisabled()
        return not beql:IsEnabled();
    end
end

beql.optionsFrames = {}

local beql_sounds = {}

-- Addon functions

-- Input: nothing
-- Output: nothing
function beql:BlizzardSetupOptions()
    -- now let's get the Options

    -- get the optiontable and register it
    beql:MainOptionsTable()
    AceConfigReg:RegisterOptionsTable(MODNAMESTRING, MainOptionsTable , MODNAMESTRING)

    -- Mainframe
    beql.optionsFrames.main = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, nil, nil, "general")

    -- Subframes
    beql.optionsFrames.questlog = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, L["Quest Log Options"], MODNAMESTRING, "qlog")
    beql.optionsFrames.questtracker = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, L["Quest Tracker Options"], MODNAMESTRING, "qtracker")
    beql.optionsFrames.tooltip = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, L["Tooltip"], MODNAMESTRING, "tooltip")
    if (beqlfu) then
        -- Fubar Plugin is loaded
        beql.optionsFrames.fubar = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, L["FubarPlugin Config"], MODNAMESTRING, "fubar")
    end
    beql.optionsFrames.profiles = AceConfigDialog:AddToBlizOptions(MODNAMESTRING, L["Profiles"], MODNAMESTRING, "profiles")
end

function getSetValue(info, ...)
    local option = beql.db;
    local path = {};

    for key in string.gmatch (info.option.dbProp or "", "[%a%d]+") do
        tinsert(path, key);
    end

    local key = path[#path];
    tremove(path, #path);

    for _, v in ipairs(path) do
        if(type(option[v]) ~= 'table') then
            return;
        end

        option = option[v];
    end

    if(select('#', ...) == 0) then
        -- get
        return type(option[key]) == 'table' and getColor(option[key]) or option[key];
    end

    if(select('#', ...) > 1) then
        setColor(option[key], ...)
    else
        option[key] = ...;
    end

    if (info.option.onSet) then
        local fnName = info.option.onSet; 
        if (type(beql[fnName]) == 'function') then
            return beql[fnName](beql);
        elseif (type(_G[fnName]) == 'function') then
            securecall(_G[fnName])
        elseif (not fnName:match('Profiles?$')) then
            beql:debugprint('function', fnName, 'not found')
        end
    end
end

function validateSetGet(t)
    if (type(t) ~= 'table') then
        return;
    end
    
    if (t.dbProp) then
        t.get = t.get or getSetValue;
        t.set = t.set or getSetValue;
    end
    
    for _, handler in ipairs({'get', 'set', 'disabled'}) do
        checkHandler(t, handler);
    end

    if (type(t.args) == 'table') then
        for _, o in pairs(t.args) do
            validateSetGet(o);
        end
    end
end

function checkHandler(t, handler)
    local fnName = t[handler];

    if (type(fnName) ~= 'string') then
        return;
    end

    if (type(beql[fnName]) == 'function') then
        t[handler] = function()
            return beql[fnName](beql);
        end
    elseif (type(_G[fnName]) == 'function') then
        t[handler] = function()
            return securecall(_G[fnName])
        end
    elseif (not fnName:match('Profiles?$')) then
        beql:debugprint('function', fnName, 'not found')
    end
end


-- Prepare the Optionstable

-- many many code ....
function beql:MainOptionsTable ()

    MainOptionsTable = {
        name = MODNAMESTRING,
        type = 'group',
        args = {

            -- General Options, shown on the main window

            general = {
                desc = L["Main Options"],
                name = L["Main Options"],
                order = 1,
                type = 'group',
                args = {
                    DisableAddon = {
                        desc = L["Enable this Addon"],
                        get = 'IsEnabled',
                        name = L["Enable Addon"],
                        order = 1,
                        set = 'ToggleActive',
                        type = 'toggle',
                        width = 'full',
                    },
                    Spacer1 = {
                        order = 2,
                        type = 'header',
                    },
                    SimpleQuestLog = {
                        desc = L["Uses the default Blizzard Quest Log"],
                        disabled = 'IsDisabled',
                        get = function(info)
                            return beql.db.profile.QuestLog.style == 'simple'
                        end,
                        name = L["Simple Quest Log"],
                        order = 3,
                        set = function(info, newval)
                            beql.db.profile.QuestLog.style = newval == true and 'simple' or 'extended';
                            beql:questLog_OnToggle()
                        end,
                        type = 'toggle',
                    },
                    DisableTracker = {
                        dbProp = "profile.disabledtracker",
                        desc = L["Disable the Tracker"],
                        disabled = 'IsDisabled',
                        name = L["Disable Tracker"],
                        order = 4,
                        onSet = 'QuestTracker_ToggleStatus',
                        type = 'toggle',
                        width = 'full',
                    },
                    Language = {
                        dbProp = "profile.EnableLanguages",
                        desc = L["Enables the language support for non-natural Languages."],
                        disabled = function(info)
                            return not beql_lang or not beql:IsEnabled()
                        end,
                        name = L["Enable Language support"],
                        order = 7,
                        type = 'toggle',
                    },
                    LanguageSelect = {
                        desc = "Change the language to a non WoW language. Reloades interface." .." - "..L["Requires Interface Reload"],
                        dbProp = "profile.Language",
                        disabled = function(info)
                            return not beql_lang or not beql.db.profile.EnableLanguages or not beql:IsEnabled()
                        end,
                        name = "Language",
                        order = 8,
                        onSet = 'ReloadUI',
                        type = 'select',
                        values = function()
                            local langtable = { SystemDefault = "SystemDefault"}
                            for k in pairs(beql_lang) do
                                langtable[k] = k
                            end
                            return langtable
                        end,
                    },
                    debug = {
                        dbProp = "profile.debug",
                        name = L["Enable Debug"],
                        type = 'toggle',
                        width = 'full',
                    },
                },
            },

            -- Quest Log Options

            qlog = {
                desc = L["Options related to the Quest Log"],
                name = L["Quest Log Options"],
                order = 2,
                type = 'group',
                args = {
                    Spacing0 = {
                        order = 0,
                        type = 'header',
                    },
                    Locked = {
                        desc = L["Makes the quest log unmovable"],
                        dbProp = "profile.QuestLog.locked",
                        disabled = 'IsDisabled',
                        name = L["Lock Quest Log"],
                        onSet = 'questLog_OnToggle',
                        order = 1,
                        type = 'toggle',
                    },
                    Reset = {
                        desc = L["Reset questlog position"],
                        func = function(info)
                            beql:restoreLayout(beql.frames.questLogAnchor);
                            beql:SaveFramePosition(beql.frames.questLogAnchor)
                            beql:questLog_OnToggle();
                        end,
                        name = L["Reset questlog position"],
                        order = 2,
                        type = 'execute',
                    },
                    Spacing1 = {
                        name = L["General Quest Log Options"],
                        order = 3,
                        type = 'header',
                    },
                    Layout = {
                        type = 'group',
                        order = 4,
                        guiInline = true,
                        name = L["Questlog Layout"],
                        disabled = 'IsDisabled',
                        args = {
                            style = {
                                desc = L["The default appearance of the quest log."],
                                dbProp = "profile.QuestLog.style",
                                disabled = 'IsDisabled',
                                name = L["Quest Log Style"],
                                onSet = 'questLog_OnToggle',
                                order = 1,
                                sorting = {'simple', 'extended' ,'minimized' ,'maximized'},
                                type = 'select',
                                values = {
                                    simple = L["Simple Quest Log"],
                                    extended = L["Extended Quest Log"],
                                    minimized = L["Always Open Minimized"],
                                    maximized = L["Always Open Maximized"],
                                }
                            },
                            Spacing1 = {
                                type = 'header',
                                order = 2,
                            },
                            ShowLevel = {
                                type = 'toggle',
                                order = 3,
                                name = L["Show Quest Level"],
                                desc = L["Shows the quests level"],
                                dbProp = "profile.QuestLog.showlevel",
                                onSet = 'QuestLog_Update',
                                disabled = 'IsDisabled',
                            },
                            ShowShortTags = {
                                type = 'toggle',
                                order = 4,
                                name = L["Show Quest Tags"],
                                desc = L["Shows the short verion of the tag"],
                                dbProp = "profile.QuestLog.showTags",
                                onSet = 'QuestLog_Update',
                                disabled = 'IsDisabled',
                            },
                            QuestLogFontSize = {
                                type = 'range',
                                order = 5,
                                width = 'full',
                                name = L["Font Size"],
                                desc = L["Font Size"],
                                dbProp = "profile.QuestLog.FontSize",
                                onSet = 'questLog_OnToggle',
                                min = 6,
                                max = 20,
                                step = 1,
                                disabled = 'IsDisabled',
                            },
                            QuestLogAlpha = {
                                type = 'range',
                                order = 6,
                                name = L["Quest Log Alpha"],
                                desc = L["Sets the Alpha of the Quest Log"],
                                dbProp = "profile.QuestLog.Alpha",
                                min = 0,
                                max = 1,
                                step = 0.01,
                                disabled = 'IsDisabled',
                            },
                            QuestLogScale = {
                                type = 'range',
                                order = 7,
                                name = L["Quest Log Scale"],
                                desc = L["Sets the Scale of the Quest Log"],
                                dbProp = "profile.QuestLog.Scale",
                                min = 0.4,
                                max = 1.5,
                                step = 0.01,
                                disabled = 'IsDisabled',
                            },
                        },
                    },
                    QuestCompletionSound = {
                        type = 'group',
                        order = 9,
                        guiInline = true,
                        width = 'full',
                        name = L["Completion Sound"],
                        desc = L["Select the sound to be played when a quest is completed"],
                        disabled = 'IsDisabled',
                        args = {
                            InfoOnQuestCompletion = {
                                type = 'toggle',
                                order = 1,
                                width = 'full',
                                name = L["Info on Quest Completion"],
                                desc = L["Shows a message and plays a sound when you complete a quest"],
                                dbProp = "profile.QuestLog.InfoOnQuestCompletion",
                                disabled = 'IsDisabled',
                            },
                            Spacing1 = {
                                type = 'header',
                                order = 2,
                                hidden = function(info)
                                    return not beql.db.profile.QuestLog.InfoOnQuestCompletion or not beql:IsEnabled()
                                end,
                            },
                            none = {
                                type = 'toggle',
                                order = 3,
                                width = 'full',
                                name = L["No sound"],
                                desc = L["Don't play a sound"],
                                get = function(info)
                                    if (not beql.db.profile.QuestLog.InfoSound or beql.db.profile.QuestLog.InfoSound ==    '') then
                                        return true
                                    else
                                        return false
                                    end
                                end,
                                hidden = function(info)
                                    return not beql.db.profile.QuestLog.InfoOnQuestCompletion or not beql:IsEnabled()
                                end,
                            },
                        },
                    },
                    AnnounceQuestStatus = {
                        type = 'group',
                        guiInline = true,
                        name = L["Quest Progression to Party Chat"],
                        order = 10,
                        args ={
                            AnnounceQuestStatus = {
                                type = 'toggle',
                                order = 1,
                                width = 'full',
                                name = L["Enable Quest Progression to Party Chat"],
                                desc = L["Prints the Quest Progression Status to the Party Chat"],
                                dbProp = "profile.QuestLog.chat.AnnounceQuest",
                                disabled = 'IsDisabled',
                            },
                            ChatType = {
                                type = 'select',
                                width = 'full',
                                order = 2,
                                name = L["Select chattype"],
                                desc = L["The chattype to post progress massages."],
                                values = {default = L["Default chat for actual group"], group = L["Group chat"], say = L["say quest progession"]},
                                dbProp = 'profile.QuestLog.chat.chattype',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                            accept = {
                                type = 'toggle',
                                order = 3,
                                width = 'full',
                                name = L["Accept a new quest"],
                                desc = L["Print a chat massage when you accept a new quest."],
                                dbProp = 'profile.QuestLog.chat.accept',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                            abandon = {
                                type = 'toggle',
                                order = 4,
                                width = 'full',
                                name = L["Abandon a quest"],
                                desc = L["Print a chat massage when you abandon a quest."],
                                dbProp = 'profile.QuestLog.chat.abandoned',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                            progress = {
                                type = 'toggle',
                                order = 5,
                                width = 'full',
                                name = L["Progess of a quest"],
                                desc = L["Print a chat massage when an objective of a quest is updated."],
                                dbProp = 'profile.QuestLog.chat.progress',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                            complete = {
                                type = 'toggle',
                                order = 6,
                                width = 'full',
                                name = L["Complete a quest"],
                                desc = L["Print a chat massage when you complete all quest objectives."],
                                dbProp = 'profile.QuestLog.chat.finished',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                            finished = {
                                type = 'toggle',
                                order = 7,
                                width = 'full',
                                name = L["Finalize a quest."],
                                desc = L["Print a chat massage when you deliver a completed quest to the NPC."],
                                dbProp = 'profile.QuestLog.chat.delivert',
                                disabled = function(info)
                                    return not beql:IsEnabled() or not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                                hidden = function()
                                    return not beql.db.profile.QuestLog.chat.AnnounceQuest
                                end,
                            },
                        },
                    },
                    AutoComplete = {
                        type = 'toggle',
                        order = 11,
                        width = 'full',
                        name = L["Auto Complete Quest"],
                        desc = L["Automatically Complete Quests"],
                        dbProp = "profile.QuestLog.autocomplete",
                        disabled = 'IsDisabled',
                    },
                },
            },

            -- Questtracker Options

            qtracker = {
                type = 'group',
                order = 3,
                name = L["Disable Tracker"],
                desc = L["Disable the Tracker"],
                args = {
                    LockedTracker = {
                        type = 'toggle',
                        order = 1,
                        name = L["Lock Tracker"],
                        desc = L["Makes the quest tracker unmovable"],
                        dbProp = "profile.QuestTracker.locked",
                        onSet = 'QuestTracker_ScheduleUpdate',
                    },
                    ResetTracker = {
                        type = 'execute',
                        order = 2,
                        name = L["Reset tracker position"],
                        desc = L["Reset tracker position"],
                        func = function(info)
                            beql.frames.questWatchAnchor:ClearAllPoints()
                            beql.frames.questWatchAnchor:SetPoint("TOPLEFT","UIParent","Center",0,0)
                            beql:SaveFramePosition(beql.frames.questWatchAnchor)
                        end,
                    },
                    Spacing1 = {
                        type = 'header',
                        order = 3,
                    },
                    style = {
                        type = 'group',
                        guiInline = true,
                        name = '[ph] style',
                        order = 4,
                        args = {
                            ShowTrackerHeader = {
                                type = 'toggle',
                                order = 4,
                                width = 'full',
                                name = L["Show Tracker Header"],
                                desc = L["Shows the trackers header"],
                                dbProp = "profile.QuestTracker.ShowHeader",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            TrackerAutoResize = {
                                type = 'toggle',
                                order = 5,
                                width = 'full',
                                name = L["Auto Resize Tracker"],
                                desc = L["Automatical resizes the tracker depending on the lenght of the text in it"],
                                dbProp = "profile.QuestTracker.AutoResize",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            TrackerFixedWidth = {
                                type = 'range',
                                order = 6,
                                width = 'full',
                                name = L["Fixed Tracker Width"],
                                desc = L["Sets the fixed width of the tracker if auto resize is disabled"],
                                dbProp = "profile.QuestTracker.FixedWidth",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                min = 20,
                                max = 600,
                                step = 10,
                                disabled = function(info)
                                    return beql.db.profile.QuestTracker.AutoResize
                                end,
                            },
                            TrackerFontHeight = {
                                type = 'range',
                                order = 7,
                                name = L["Font Size"],
                                desc = L["Changes the font size of the tracker"],
                                dbProp = "profile.QuestTracker.FontHeight",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                min = 8,
                                max = 18,
                                step = 1,
                            },
                            QuestTrackerAlpha = {
                                type = 'range',
                                order = 8,
                                name = L["Quest Tracker Alpha"],
                                desc = L["Sets the Alpha of the Quest Tracker"],
                                dbProp = "profile.QuestTracker.Alpha",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                min = 0,
                                max = 1,
                                step = 0.01,
                            },
                        },
                    },
                    WatchOptions = {
                        type = 'group',
                        guiInline = true,
                        name = L["Watch Options"],
                        order = 5,
                        args = {
                            AddNew = {
                                type = 'toggle',
                                order = 10,
                                width = 'full',
                                name = L["Add New Quests"],
                                desc = L["Automatical add new Quests to tracker"],
                                dbProp = "profile.QuestTracker.AddNew",
                            },
                            AddUntracked = {
                                type = 'toggle',
                                order = 11,
                                width = 'full',
                                name = L["Add Untracked"],
                                desc = L["Automatical add quests with updated objectives to tracker"],
                                dbProp = "profile.QuestTracker.AddUntracked",
                            },
                            RemoveFinished = {
                                type = 'toggle',
                                order = 15,
                                width = 'full',
                                name = L["Remove Completed Quests"],
                                desc = L["Automatical remove completed quests from tracker"],
                                dbProp = "profile.QuestTracker.RemoveFinished",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            MinimizeFinished = {
                                type = 'toggle',
                                order = 16,
                                width = 'full',
                                name = L["Hide Completed Objectives"],
                                desc = L["Automatical hide completed objectives in tracker"],
                                dbProp = "profile.QuestTracker.MinimizeFinished",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            HideCompletedOnly = {
                                type = 'toggle',
                                order = 17,
                                width = 'full',
                                name = L["Hide Objectives for Completed only"],
                                desc = L["Hide objectives only for completed quests"],
                                dbProp = "profile.QuestTracker.HideCompletedOnly",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                disabled = function(info)
                                    return beql.db.profile.QuestTracker.MinimizeFinished or beql.db.profile.QuestTracker.RemoveFinished
                                end,
                            },
                            SortTrackerItems = {
                                type = 'toggle',
                                order = 18,
                                width = 'full',
                                name = L["Sort Tracker Quests"],
                                desc = L["Sort the quests in tracker"],
                                dbProp = "profile.QuestTracker.SortItems",
                                onSet = 'QuestTracker_SortWatchedQuests() beql:QuestTracker_ScheduleUpdate',
                            },
                        },
                    },

                    -- Colors on Questtracker

                    Colors = {
                        type = 'group',
                        guiInline = true,
                        name = L["Colors"],
                        order = 22,
                        args = {
                            UseBackgroundColor = {
                                type = 'toggle',
                                order = 1,
                                name = L["Background"],
                                desc = L["Use Background"],
                                dbProp = "profile.QuestTracker.UseBgColors",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            CustomBgColor = {
                                type = 'toggle',
                                order = 2,
                                name = L["Custom Background Color"],
                                desc = L["Use custom color for background"],
                                dbProp = "profile.QuestTracker.CustomBgColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.UseBgColors
                                end,
                            },
                            BackgroundColor = {
                                type = 'color',
                                order = 3,
                                name = L["Background Color"],
                                desc = L["Sets the Background Color"],
                                dbProp = "profile.QuestTracker.Colors.Bg",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = true,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomBgColor
                                end,
                            },
                            QuestTracker_BgBorder = {
                                type = 'color',
                                order = 4,
                                name = L["Background Corner Color"],
                                desc = L["Sets the Background Corner Color"],
                                dbProp = "profile.QuestTracker.Colors.BgBorder",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = true,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomBgColor
                                end,
                            },
                            spacer1 = {
                                type = 'header',
                                order = 5,
                                name = L["Zones"],
                            },
                            ShowZonesInTracker = {
                                order = 5.5,
                                type = 'toggle',
                                width = 'full',
                                name = L["Show Quest Zones"],
                                desc = L["Show the quests zone it belongs to above its name"],
                                dbProp = "profile.QuestTracker.ShowZones",
                                onSet = 'QuestTracker_SortWatchedQuests() beql:QuestTracker_ScheduleUpdate',
                            },
                            CustomZoneColor = {
                                type = 'toggle',
                                order = 6,
                                name = L["Custom Zone Color"],
                                desc = L["Use custom color for Zone names"],
                                dbProp = "profile.QuestTracker.CustomZoneColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                disabled = function()
                                    return not beql.db.profile.QuestTracker.ShowZones
                                end,
                            },
                            ColorZone = {
                                type = 'color',
                                order = 7,
                                name = L["Zone Color"],
                                desc = L["Sets the zone color"],
                                dbProp = "profile.QuestTracker.Colors.Zone",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomZoneColor or not beql.db.profile.QuestTracker.ShowZones
                                end,
                            },
                            spacer3 = {
                                type = 'header',
                                order = 8,
                            },
                            CustomHeaderColor = {
                                type = 'toggle',
                                order = 9,
                                name = L["Custom Header Color"],
                                desc = L["Use custom color for headers"],
                                dbProp = "profile.QuestTracker.CustomHeaderColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            ColorHeaderEmpty = {
                                type = 'color',
                                order = 10,
                                name = L["Header Color"],
                                desc = L["Sets the color for headers"],
                                dbProp = "profile.QuestTracker.Colors.HeaderNormal",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomHeaderColor or beql.db.profile.QuestTracker.HeaderQuestLevelColor
                                end,
                            },
                            HeaderQuestLevelColor = {
                                type = 'toggle',
                                width = 'full',
                                order = 11,
                                name = L["Use Quest Level Colors"],
                                desc = L["Use the colors to indicate quest difficulty"],
                                dbProp = "profile.QuestTracker.HeaderQuestLevelColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomHeaderColor
                                end,
                            },
                            ColorHeaderFailed = {
                                type = 'color',
                                order = 12,
                                name = L["Failed Header Color"],
                                desc = L["Sets the color for failed quests"],
                                dbProp = "profile.QuestTracker.Colors.HeaderFailed",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomHeaderColor
                                end,
                            },
                            ColorHeaderComplete = {
                                type = 'color',
                                order = 13,
                                name = L["Completed Header Color"],
                                desc = L["Sets the color for completed headers"],
                                dbProp = "profile.QuestTracker.Colors.HeaderComplete",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomHeaderColor
                                end,
                            },
                            spacer2 = {
                                type = 'header',
                                order = 14,
                            },
                            CustomObjetiveColor = {
                                type = 'toggle',
                                order = 15,
                                name = L["Color Objectives"],
                                desc = L["Use custom color for objective text"],
                                dbProp = "profile.QuestTracker.CustomObjetiveColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            FadeColor = {
                                type = 'toggle',
                                order = 16,
                                name = L["Fade Colors"],
                                desc = L["Fade the objective colors"],
                                dbProp = "profile.QuestTracker.FadeColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            ColorObjectiveNormal = {
                                type = 'color',
                                order = 17,
                                name = L["Objective Color"],
                                desc = L["Sets the color for objectives"],
                                dbProp = "profile.QuestTracker.Colors.ObjectiveNormal",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomObjetiveColor
                                end,
                            },
                            ColorObjectiveComplete = {
                                type = 'color',
                                order = 18,
                                name = L["Completed Objective Color"],
                                desc = L["Sets the color for completed objectives"],
                                dbProp = "profile.QuestTracker.Colors.ObjectiveComplete",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomObjetiveColor
                                end,
                            },
                            ObjectiveStepColor = {
                                type = 'toggle',
                                order = 19,
                                name = L["Custom Objective Color"],
                                desc = L["Use custom color for objective text"],
                                dbProp = "profile.QuestTracker.ObjetiveStepColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            ColorObjectiveStep = {
                                type = 'color',
                                order = 20,
                                name = L["Completed Objective Color"],
                                desc = L["Sets the color for completed objectives"],
                                dbProp = "profile.QuestTracker.Colors.ObjetiveStepColor",
                                onSet = 'QuestTracker_ScheduleUpdate',
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.QuestTracker.CustomObjetiveColor or not beql.db.profile.QuestTracker.ObjetiveStepColor
                                end,
                            },
                            MarkerOptions = {
                                type = 'group',
                                order = 21,
                                guiInline = true,
                                desc = L["Customize the Objective/Quest Markers"],
                                args = {
                                    Spacing5 = {
                                        type = 'header',
                                        order = 0,
                                        name = L["Markers"],
                                    },
                                    ShowObjectiveMarkers = {
                                        type = 'toggle',
                                        order = 1,
                                        name = L["Show Objective Markers"],
                                        desc = L["Display Markers before objectives"],
                                        dbProp = "profile.QuestTracker.ShowObjectiveMarkers",
                                        onSet = 'QuestTracker_ScheduleUpdate',
                                    },
                                    TrackerSymbol = {
                                        type = 'range',
                                        order = 2,
                                        name = L["Symbol Type"],
                                        desc = L["Set the type of symbol"],
                                        dbProp = "profile.QuestTracker.TrackerSymbol",
                                        onSet = 'QuestTracker_ScheduleUpdate',
                                        min = 1,
                                        max = 4,
                                        step = 1,
                                        disabled = function(info)
                                            return beql.db.profile.QuestTracker.UseTrackerListing or not beql.db.profile.QuestTracker.ShowObjectiveMarkers
                                        end,
                                    },
                                    UseTrackerListing = {
                                        type = 'toggle',
                                        order = 3,
                                        name = L["Use Listing"],
                                        desc = L["User Listing rather than symbols"],
                                        dbProp = "profile.QuestTracker.UseTrackerListing",
                                        onSet = 'QuestTracker_ScheduleUpdate',
                                        disabled = function(info)
                                            return not beql.db.profile.QuestTracker.ShowObjectiveMarkers
                                        end,
                                    },
                                    TrackerList = {
                                        type = 'range',
                                        order = 4,
                                        name = L["List Type"],
                                        desc = L["Set the type of listing"],
                                        dbProp = "profile.QuestTracker.TrackerList",
                                        onSet = 'QuestTracker_ScheduleUpdate',
                                        min = 1,
                                        max = 4,
                                        step = 1,
                                        disabled = function(info)
                                            return not beql.db.profile.QuestTracker.UseTrackerListing
                                        end,
                                    },
                                },
                            },
                        },
                    },
                },
            },

            -- Tooltip Options

            tooltip = {
                type = 'group',
                order = 4,
                name = L["Tooltip"],
                desc = L["Tooltip Options"],
                args = {
                    Spacer1 = {
                        type = 'header',
                        order = 3,
                        name = L["Quest Tracker"],
                    },
                    QuestTrackerTooltip = {
                        type = 'group',
                        name = L["Quest Tracker"],
                        order = 3,
                        guiInline = true,
                        args = {
                            ActiveTracker = {
                                type = 'toggle',
                                order = 1,
                                width = 'full',
                                name = L["Tracker Tooltip"],
                                desc = L["Showing mouseover tooltips in tracker"],
                                dbProp = "profile.Tooltip.QuestTracker_TooltipShow",
                                onSet = 'QuestTracker_ScheduleUpdate',
                            },
                            TrackerMouseLeft = {
                                type = 'toggle',
                                order = 3,
                                name = L["Enable Left Click"],
                                desc = L["Left clicking a quest in the tracker opens the Quest Log"],
                                dbProp = "profile.QuestTracker.LeftClick",
                            },
                            TrackerMouseRight = {
                                type = 'toggle',
                                order = 4,
                                name = L["Enable Right Click"],
                                desc = L["Right clicking a quest in the tracker removes it from the tracker"],
                                dbProp = "profile.QuestTracker.RightClick",
                            },
                            Spacer2 = {
                                type = 'header',
                                order = 5,
                                name = L["Title color"],
                            },
                            ColorTitleDiff = {
                                type = 'toggle',
                                order = 6,
                                name = L["Use Quest Level Colors"],
                                desc = L["Use Quest Level Colors"],
                                dbProp = "profile.Tooltip.QuestTracker_TooltipTitleQuestcolor",
                            },
                            ColorTitle = {
                                type = 'color',
                                order = 7,
                                name = L["Header Color"],
                                desc = L["Sets the color for headers"],
                                dbProp = "profile.Tooltip.Colors.QuestTracker_TooltipTitle",
                                hasAlpha = false,
                                disabled = function(info)
                                    return beql.db.profile.Tooltip.QuestTracker_TooltipTitleQuestcolor
                                end,
                            },
                            TrackerTooltipDesc = {
                                type = 'toggle',
                                order = 8,
                                width = 'full',
                                name = L["Quest Description in Tracker Tooltip"],
                                desc = L["Displays the actual quest's description in the tracker tooltip"],
                                dbProp = "profile.Tooltip.QuestTracker_TooltipDesc",
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.QuestTracker_TooltipShow
                                end,
                            },
                            ColorDesc = {
                                type = 'color',
                                order = 9,
                                name = L["Quest Description Color"],
                                desc = L["Sets the color for the Quest description"],
                                dbProp = "profile.Tooltip.Colors.QuestTracker_TooltipDesc",
                                hasAlpha = false,
                            },
                            TrackerTooltipParty = {
                                type = 'toggle',
                                order = 10,
                                width = 'full',
                                name = L["Party Quest Progression info"],
                                desc = L["Displays Party members quest status in the tooltip - Quixote must be installed on the partymembers client"],
                                dbProp = "profile.activetrackerparty",
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.QuestTracker_TooltipShow
                                end,
                                hidden = true,
                            },
                        },
                    },
                    ColorPartyQ = {
                        type = 'color',
                        order = 2,
                        width = 'full',
                        name = L["Party Member with Quixote Color"],
                        desc = L["Sets the color for Party member"],
                        dbProp = "profile.Color.TooltipPartyQuixote",
                        hasAlpha = false,
                        hidden = true,
                    },
                    ColorPartyNQ = {
                        type = 'color',
                        order = 3,
                        width = 'full',
                        name = L["Party Member Color"],
                        desc = L["Sets the color for Party member"],
                        dbProp = "profile.Color.TooltipPartyNonQuixote",
                        hasAlpha = false,
                        hidden = true,
                    },
                    ColorPartyObj = {
                        type = 'color',
                        order = 4,
                        width = 'full',
                        name = L["Objective Color"],
                        desc = L["Sets the color for objectives"],
                        dbProp = "profile.Color.TooltipPartyObj",
                        hasAlpha = false,
                        hidden = true,
                    },
                    ColorPartyObj = {
                        type = 'color',
                        order = 5,
                        width = 'full',
                        name = L["Completed Objective Color"],
                        desc = L["Sets the color for completed objectives"],
                        dbProp = "profile.Color.TooltipPartyObjComp",
                        hasAlpha = false,
                        hidden = true,
                    },
                    ColorObjFade = {
                        type = 'toggle',
                        order = 7,
                        width = 'full',
                        name = L["Fade Colors"],
                        desc = L["Fade Colors"],
                        dbProp = "profile.TooltipObjFade",
                        hidden = true,
                    },
                    Spacer1 = {
                        type = 'header',
                        order = 10,
                        name = L["NPC color"],
                    },
                    Gametooltip = {
                        type = 'group',
                        order = 14,
                        guiInline = true,
                        name = L["GameTooltip"],
                        desc = L["Colors"],
                        args = {
                            TooltipMob = {
                                type = 'toggle',
                                order = 1,
                                width = 'full',
                                name = L["Mob Tooltip Quest Info"],
                                desc = L["Show quest info in mob tooltips"],
                                dbProp = "profile.Tooltip.mob",
                            },
                            TooltipItem = {
                                type = 'toggle',
                                order = 2,
                                width = 'full',
                                name = L["Item Tooltip Quest Info"],
                                desc = L["Show quest info in item tooltips"],
                                dbProp = "profile.Tooltip.item",
                            },
                            TooltipTitleColor = {
                                type = 'toggle',
                                order = 3,
                                width = 'full',
                                name = L["Use Quest Level Colors"],
                                desc = L["Use Quest Level Colors"],
                                dbProp = "profile.Tooltip.GameTooltip_TitelIsQuestcolor",
                            },
                            ColorMobTooltip = {
                                type = 'color',
                                order = 4,
                                name = L["Mob Tooltip Quest Info"],
                                desc = L["Mob Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_MobTitle",
                                hasAlpha = false,
                                disabled = function(info)
                                    return beql.db.profile.Tooltip.GameTooltip_TitelIsQuestcolor
                                end,
                            },
                            ColorItemTooltip = {
                                type = 'color',
                                order = 5,
                                name = L["Item Tooltip Quest Info"],
                                desc = L["Item Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_ItemTitle",
                                hasAlpha = false,
                                disabled = function(info)
                                    return beql.db.profile.Tooltip.GameTooltip_TitelIsQuestcolor
                                end,
                            },
                            Spacer1 = {
                                type = 'header',
                                order = 6,
                            },
                            ColorObjectives = {
                                type = 'toggle',
                                order = 7,
                                name = L["Color Objectives"],
                                -- desc = L["Show quest info in item tooltips"],
                                dbProp = "profile.Tooltip.GameTooltip_ColorObjectives",
                            },
                            FadeObjectives = {
                                type = 'toggle',
                                order = 8,
                                name = L["Fade Colors"],
                                -- desc = L["Show quest info in item tooltips"],
                                dbProp = "profile.Tooltip.GameTooltip_FadeObjectiveColors",
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives
                                end,
                            },
                            ColorIncomplete = {
                                type = 'color',
                                order = 9,
                                name = L["Incomplete"],
                                -- desc = L["Item Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_ObjectiveIncomplete",
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives
                                end,
                            },
                            ColorComplete = {
                                type = 'color',
                                order = 10,
                                name = L["Complete"],
                                -- desc = L["Item Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_ObjectiveComplete",
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives
                                end,
                            },
                            UseFadeStepColor = {
                                type = 'toggle',
                                order = 11,
                                name = L["Use Fade-Step-Color"],
                                -- desc = L["Show quest info in item tooltips"],
                                dbProp = "profile.Tooltip.GameTooltip_ObjectiveUseStep",
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives or not beql.db.profile.Tooltip.GameTooltip_FadeObjectiveColors
                                end,
                            },
                            ColorFadeStep = {
                                type = 'color',
                                order = 12,
                                name = L["Fade-Step-Color"],
                                -- desc = L["Item Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_ObjectiveStep",
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives or not beql.db.profile.Tooltip.GameTooltip_ObjectiveUseStep or not beql.db.profile.Tooltip.GameTooltip_FadeObjectiveColors
                                end,
                            },
                            ColorNonobjective = {
                                type = 'color',
                                order = 13,
                                name = L["Non Objective"],
                                -- desc = L["Item Tooltip Quest Info"],
                                dbProp = "profile.Tooltip.Colors.GameTooltip_NonObjective",
                                hasAlpha = false,
                                disabled = function(info)
                                    return not beql.db.profile.Tooltip.GameTooltip_ColorObjectives
                                end,
                            },
                        },
                    },
                },
            },

            -- Profiles
            profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
        },
    }

    -- add Sounds to Options

    local i = 1
    for k, v in pairs (beql_sounds) do
        i = i + 1
        MainOptionsTable.args.qlog.args.QuestCompletionSound.args[k] = {
            type = 'toggle',
            order = i + 3,
            name = k,
            desc = k,
            get = function(info)
                return beql.db.profile.QuestLog.InfoSound == v;
            end,
            set = function(info, newval)
                if (newval) then
                    beql.db.profile.QuestLog.InfoSound = v
                    PlaySoundFile(v)
                end
            end,
            hidden = function(info)
                return not beql.db.profile.QuestLog.InfoOnQuestCompletion or not beql:IsEnabled()
            end
        }
    end

    validateSetGet(MainOptionsTable);
end

-- Save postion of a frame
function beql:SaveFramePosition(frame)
    local name, posx, posy, point;

    for n, f in pairs(beql.frames) do
        if (frame == f) then
            name = n;
        end
    end

    if (not name) then
        return;
    end

    local x, y = frame:GetCenter()
    local hhalf = (x > UIParent:GetWidth()/2) and "RIGHT" or "LEFT"
    local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
    point = vhalf .. hhalf

    posx = beql:round(hhalf == "LEFT" and (frame:GetLeft() * UIParent:GetScale()) or ((frame:GetLeft() - UIParent:GetWidth()) * UIParent:GetScale()) , 2)
    posy = beql:round(vhalf == "TOP" and ((frame:GetTop() - UIParent:GetHeight()) * UIParent:GetScale()) or (frame:GetTop() * UIParent:GetScale()) , 2)

    frame:ClearAllPoints()
    frame:SetPoint("TOPLEFT", UIParent, point, posx/UIParent:GetScale(), posy/UIParent:GetScale())

    beql.db.char.FramePositions[name] = {
        x = posx,
        y = posy,
        point = point,
    };
end

-- restore position of a frame
function beql:RestoreFramePosition(frame)
    local name, info;
    for n, f in pairs(beql.frames) do
        if (frame == f) then
            name = n;
        end
    end
    
    if (not name) then
        return;
    end

    frame.UIScale = UIParent:GetScale();
    info = beql.db.char.FramePositions[name];

    if (not info) then
        return;
    end

    frame:ClearAllPoints()
    frame:SetPoint("TOPLEFT", UIParent, info.point, info.x/UIParent:GetScale(), info.y/UIParent:GetScale())

    beql:debugprint("Restore postion of frame: " .. name)
end

-- open Optionsinterface for a category
function beql:OpenBlizzardOption(option)
    if (not option) or (not beql.optionsFrames[option]) then
        assert(false, "Option " .. option .. " not found in options table!")
    end

    InterfaceOptionsFrame_OpenToCategory(beql.optionsFrames[option])
end

-- Internal for Compatiblity with other addons
-- #NODOC
function beql:Compatibility()
    do
        return
    end

    if (IsAddOnLoaded("Mobmap")) then
        beql.db.profile.disabled.markers = true
        MainOptionsTable.args.qtracker.args.MarkerOptions.desc = MainOptionsTable.args.qtracker.args.MarkerOptions.desc.." "..L[" |cffff0000Disabled by|r"].." |cffe0c000Mobmap|r"
        MainOptionsTable.args.qtracker.args.MarkerOptions.args.ShowObjectiveMarkers.desc = MainOptionsTable.args.qtracker.args.MarkerOptions.args.ShowObjectiveMarkers.desc.." "..L[" |cffff0000Disabled by|r"].." |cffe0c000Mobmap|r"
        if (beql.db.profile.QuestTracker.ShowObjectiveMarkers) then
            beql.db.profile.QuestTracker.ShowObjectiveMarkers = false
        end
    else
        beql.db.profile.disabled.markers = false
    end

    if (QuestLogFrame_MidTextures) ~= nil then
        MainOptionsTable.args.qlogoption.args.SimpleQuestLog.desc = MainOptionsTable.args.qlogoption.args.SimpleQuestLog.desc.." "..L[" |cffff0000Disabled by|r"].." |cffe0c000FramesResized|r"
        beql.db.profile.disabled.style = true
        beql.db.profile.style = true
    else
        beql.db.profile.disabled.style = false
    end

    if (not IsAddOnLoaded("FuBar")) then
        beql.db.profile.fubar.FubarPosition = "MINIMAP"
    end
end

-- Tracker List/Symbol and Soundtable
BEQL_TrackerLists = {
    {1, 2, 3, 4, 5, 6, 7, 8, 9, 0},
    {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"},
    {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J"},
    {"I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX", "X"}
}

BEQL_TrackerSymbols = {"-", "+", "@", ">"}

beql_sounds = {
    ["Exploration"] = "Sound\\Interface\\GnomeExploration.wav",
    ["HordeVictory"] = "Sound\\Interface\\PVPVictoryHordeMono.wav",
    ["AllianceVictory"] = "Sound\\Interface\\PVPVictoryAllianceMono.wav",
    ["FlagCaptured1"] = "Sound\\Interface\\PVPFlagCapturedmono.wav",
    ["FlagCaptured2"] = "Sound\\Interface\\PVPFlagCapturedHordemono.wav",
    ["RaidWarning"] = "Sound\\Interface\\RaidWarning.wav",
    ["ReadyCheck"] = "Sound\\Interface\\ReadyCheck.wav",
    ["LevelUp1"] = "Sound\\Interface\\LevelUp.wav",
    ["LevelUp2"] = "Sound\\Interface\\levelup2.wav",
    ["Message"] = "Sound\\Interface\\iTellMessage.wav",
    ["Cow"] = "Sound\\Creature\\Cow\\CowDeath.wav",
    ["WorkComplete"] = "Sound\\Creature\\Peon\\PeonBuildingComplete1.wav"
}

function getColor(input)
    if (type(input) == 'table') then
        return input.r, input.g, input.b, input.a;
    end
end

function setColor(target, r, g, b, a)
    if (type(target) == 'table') then
        target.r = r;
        target.g = g;
        target.b = b;
        target.a = a;
    end
end

-- - EOF -- -
