-- Author      : admin
-- Create Date : 11/7/2019 12:58:09 PM

local addOnName = ...;

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local quests, objectives, changes = {}, {}, {};
local scan, scanning, deepScan, fireEvents;
local initialized, eventFrame = false, CreateFrame('Frame');
local cState = {[-1] = 'failed', [0] = 'incompleted', [1] = 'completed'};

local pairs = _G.pairs;
local tinsert = _G.tinsert;
local tonumber = _G.tonumber;
local type = _G.type;
local GetNumQuestLeaderBoards = _G.GetNumQuestLeaderBoards;
local GetNumQuestLogEntries = _G.GetNumQuestLogEntries;
local GetQuestLogLeaderBoard = _G.GetQuestLogLeaderBoard;
local GetQuestLogTitle = _G.GetQuestLogTitle;

--[[
--------------------------------------------
-- Notes:
-- Scan for quest log changes
--
-- Returns:
-- * nothing
--------------------------------------------]]
function scan()
    if (scanning) then
        return;
    end

    scanning = true;

    local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID, lastHeader, key, quest, objective;
    local visibleQuests = GetNumQuestLogEntries();
    local questIndex = 1;

    if (type(quests) == 'table') then
        for _, quest in pairs(quests) do
            quest.index = nil;
            if (quest.timerIndex ~= nil) then
                quest.timerIndex = -1;
            end
        end
    end

    local timers = {};
    for idx in ipairs({GetQuestTimers()}) do
        local timerIndex = GetQuestIndexForTimer(idx);
        timers[timerIndex] = idx;
    end

    while (true) do
        questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(questIndex);

        if (questLogTitleText == nil) then
            break;
        end

        key = isHeader and questLogTitleText or questID;
        lastHeader = isHeader and questLogTitleText or lastHeader;
        isComplete = isComplete or 0;

        if (questIndex > visibleQuests) then
            lastHeader = '_UNKNOWN';
        end

        if (quests[key] == nil) then
            -- new item
            quests[key] = {
                header = lastHeader,
                title = questLogTitleText,
                status = 0,
                objectives = {};
            };

            tinsert(changes, 'q:' .. key .. ':added');
        end

        quest = quests[key];

        quest.index = questIndex;
        quest.level = level;

        if (timers[questIndex] ~= nil) then
            if (quest.timerIndex == nil) then
                tinsert(changes, 'q:' .. key .. ':timer_added');
            end

            quest.timerIndex = timers[questIndex];
        end

        if (quest.status ~= isComplete) then
            quest.status = isComplete;
            tinsert(changes, 'q:' .. key .. ':' .. cState[isComplete]);
        end

        for i = 1, GetNumQuestLeaderBoards(questIndex) do
            local oText, oType, oComplete = GetQuestLogLeaderBoard(i, questIndex);

            -- only add objectives, wehn data is fully loaded
            if (not oText:match('^[ :]') and strlen(oText) > 0) then
                key = 'q:' .. questID .. ':o:' .. i;

                if (objectives[key] == nil) then
                    objectives[key] = {
                        completed = false,
                        countable = #{oText:match('(%d+)/(%d+)')} > 0,
                        got = '0',
                        id = questID,
                        index = i,
                        need = '0',
                        type = oType
                    };

                    tinsert(changes, key .. ':added');
                end

                objective = objectives[key];

                if (objective.countable) then
                    local got, need = oText:match('(%d+)/(%d+)');

                    if (objective.got ~= got) then
                        tinsert(changes, key .. ':changed:' .. (objective.got or 0) .. ':' .. got);
                        objective.got = got
                    end

                    objective.need = need;
                end

                if (objective.completed ~= oComplete) then
                    objective.completed = oComplete;
                    tinsert(changes, key .. ':' .. (oComplete and 'completed' or 'incompleted'));
                end
            end
        end

        questIndex = questIndex + 1;
    end

    for questID, quest in pairs(quests) do
        if (quest.timerIndex == -1) then
            tinsert(changes, 'q:' .. questID .. ':timer_removed');
        end

        if (quest.index == nil) then
            tinsert(changes, 'q:' .. questID .. ':removed');
            quests[questID] = nil;

            for objectiveID in pairs(objectives) do
                if(objectiveID:match('q:' .. questID .. ':o')) then
                    tinsert(changes, objectiveID .. ':removed');
                    objectives[objectiveID] = nil;
                end
            end
        end

        if (quest.header == '_UNKNOWN') then
            deepScan();
        end
    end

    scanning = false;

    fireEvents();
end

--[[
--------------------------------------------
-- Notes:
-- Makes a deep scan for hiden quests to get
-- her header
--
-- Returns:
-- * nothing
--------------------------------------------]]
function deepScan()
    QuestLogFrame:UnregisterEvent('QUEST_LOG_UPDATE');
    local header = {};
    local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID, lastHeader;
    local cnt = GetNumQuestLogEntries();
    
    for questIndex = 1, cnt do
        questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(questIndex);

        if (isHeader and isCollapsed) then
            tinsert(header, questIndex);
        end
    end
    
    ExpandQuestHeader(0);
    cnt = GetNumQuestLogEntries();

    for questIndex = 1, cnt do
        questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(questIndex);

        lastHeader = isHeader and questLogTitleText or lastHeader;

        if (questID > 0) then
            quests[questID].header = lastHeader;
        end
    end
    
    for _, headerIndex in ipairs(header) do
        CollapseQuestHeader(headerIndex);
    end

    QuestLogFrame:RegisterEvent('QUEST_LOG_UPDATE');
end

--[[
--------------------------------------------
-- Notes:
-- Fires event with callback handler
--
-- Returns:
-- * nothing
--------------------------------------------]]
function fireEvents()
    if (#changes == 0) then
        return;
    end

    for _, change in pairs(changes) do
        local t = {};

        for w in change:gmatch('([^:]+)') do 
            tinsert(t, tonumber(w) or w) 
        end

        if (#t == 3 and tonumber(t[2]) == nil) then
            -- quest: event, headerName
            beql.callbacks:Fire('HEADER_' .. strupper(t[3]), t[2]);
            beql:debugprint('HEADER_' .. strupper(t[3]), t[2]);
        elseif (#t == 3) then
            -- quest: event, questID
            beql.callbacks:Fire('QUEST_' .. strupper(t[3]), t[2]);
            beql:debugprint('QUEST_' .. strupper(t[3]), t[2]);
        elseif (t[1] == 'q' and t[3] == 'o') then
            -- objectives: event, questID, objectiveIndex[, oldvalue, newvalue]
            beql.callbacks:Fire('OBJECTIVE_' .. strupper(t[5]), t[2], t[4], t[6], t[7]);
            beql:debugprint('OBJECTIVE_' .. strupper(t[5]), t[2], t[4], t[6], t[7]);
        else
            beql:debugprint('unknown', unpack(t));
        end
    end

    beql.callbacks:Fire('QUEST_LOG_CHANGED');

    changes = {};
end

--[[
--------------------------------------------
-- Notes:
-- Gets Quest information like GetQuestLogTitle
--
-- Returns:
--------------------------------------------]]
function beql:GetQuestLogTitle(questID)
    if (quests[questID] == nil) then
        return GetQuestLogTitle(0);
    end

    local questData = {GetQuestLogTitle(quests[questID].index)};

    while (#questData > 8) do
        tremove(questData, 9);
    end

    tinsert(questData, quests[questID].index);
    tinsert(questData, quests[questID].header);

    return unpack(questData);
end

--[[
--------------------------------------------
-- Notes:
-- Gets Quest objective information like GetQuestLogLeaderBoard
--
-- Returns:
--------------------------------------------]]
function beql:GetQuestLogObjective(questID, index)
    if (quests[questID] == nil) then
        return;
    end

    local key = 'q:' .. questID .. ':o:' .. index;
    local oText, oType, oComplete = GetQuestLogLeaderBoard(index, quests[questID].index);

    return oText, oType, oComplete, objectives[key].got, objectives[key].need;
end

function beql:IsQuestValid(questID)
    return quests[questID] ~= nil;
end

--[[
--------------------------------------------
-- Notes:
-- Gets all current quistids for the zone
--
-- Returns:
-- * table
--------------------------------------------]]
function beql:GetZoneQuests(questZone)
    local questIDs = newArray();

    for questID, quest in pairs (quests) do
        if (quest.header == questZone and questID ~= questZone) then
            questIDs.push(questID);
        end
    end

    return questIDs;
end

--[[
--------------------------------------------
-- Notes:
-- Check for specific Quest timer
--
-- Returns:
-- * boolean
--------------------------------------------]]
function beql:HasQuestTimer(questID)
    return quests[questID] ~= nil and quests[questID].timerIndex ~= nil;
end

--[[
--------------------------------------------
-- Notes:
-- Get time lift for a Quest timer
--
-- Returns:
-- * number
--------------------------------------------]]
function beql:GetQuestTime(questID)
    if(beql:HasQuestTimer(questID) == false) then
        return;
    end

    local timer = {GetQuestTimers()};

    return timer[quests[questID].timerIndex];
end

scan();

eventFrame:RegisterEvent('QUEST_LOG_UPDATE');
eventFrame:SetScript('OnEvent', scan);

-- EOF --
